# -*- coding: UTF-8 -*-
#
# Copyright (C) 2009 - Phyware Chen
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2, or (at your option)
# any later version.
#
# The Rhythmbox authors hereby grant permission for non-GPL compatible
# GStreamer plugins to be used and distributed together with GStreamer
# and Rhythmbox. This permission is above and beyond the permissions granted
# by the GPL license by which Rhythmbox is covered. If you modify this code
# you may extend this exception to your version of the code, but you are not
# obligated to do so. If you do not wish to do so, delete this exception
# statement from your version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA.

import re
import codecs
import sys
import urllib
import os.path

class BaiduLrc:
	def __init__(self, plugin):
		self.plugin = plugin
		self.callback = plugin.lrcGot

		self.pattern = re.compile(u"文件格式" + '.*?LRC/Lyric.*?' + 'a href="(?P<url>.+?)".*?' + u'HTML版')
		self.patDiv = re.compile(r'<div style="position:\s*relative;*">')
		self.patBr = re.compile(r'<[Bb][Rr]>')	#<br>
		self.patDel = re.compile(r'<.*?>')	#pattern to delete
		self.patWhite = re.compile(r'^\s*')	#delete whitespace character at the beginning of a line
		self.patSplit = re.compile('\n')

		#timeout = 10	#have switched to use rb.Loader().get_url()
		#socket.setdefaulttimeout(timeout)
		#self.user_agent = 'Mozilla/4.0 (compatible; MSIE 5.5; Windows NT)'
		#self.headers = { 'User-Agent' : self.user_agent }
		
	def getLrc(self, title, artist, dir):
		self.dir =dir
		#the name to search is different from the name to store
		songNameList =[artist + ' - ' + title, title]	#the name to store
		if artist == 'Unknown':
			artist = ''
		else:
			artist = ' - ' + artist 
		nameList = [title + artist , title]	#the name to search

		name = None
		canEncoded = False
		index = 0
		for name in nameList:
			try:
				name = name.encode('gbk')
			except UnicodeEncodeError:
				index = index + 1
				continue
			else:
				canEncoded = True
				break

		if canEncoded == False:
			self.plugin.showState('Can\'t encode song information')
			return
		self.songName = songNameList[index]
		data = {}
		data['wd'] = name + ' filetype:lrc'
	
		urlValue = urllib.urlencode(data)
		url = 'http://www.baidu.com/s?' + urlValue

		self.plugin.get_url(url, self.htmlCB)

	def htmlCB(self, data):		#html file from 'http://www.baidu.com/s?'
		if data == None:
			self.plugin.showState('can\'t connect to the Baidu Server')
			return
		self.urlList = self.getUrl(data)
		self.length = len(self.urlList)
		self.i = 0
		self.lrcCB(None)	#fetch the lrc File
		
	def lrcCB(self, data):
		if data != None:
			songFile = open(os.path.join(self.dir, self.songName + '.lrc'), 'w')
			data = re.split(self.patSplit, data)
			index = 0
			for line in data:
				index = index + 1
				if self.patDiv.search(line) != None:
					break
			for line in data[index:]:
				line = self.patBr.sub('\n', line)
				line = self.patDel.sub('', line)
				line = self.patWhite.sub('',line)
				if(line == ''):
					continue
				songFile.write(line)
			fileName = songFile.name
			songFile.close()
			if self.callback(fileName) == True:
				return

		if self.i >= self.length:	#all lrc links have been checked
			self.plugin.showState('can\'t download valid lrc file')
			return
		url = self.urlList[self.i]
		self.i = self.i + 1
		self.plugin.get_url(url, self.lrcCB)
			
	def getUrl(self, data):
		url = []
		data = data.decode('gb2312')
		data = re.split(self.patSplit, data)
		for line in data:
			result = self.pattern.findall(line)
			url = url + result
		return url
		

#for testing		
if __name__ == "__main__":
	lrc = BaiduLrc(None)
	lrc.dir = ''
	lrc.songName = 'test'
	file = open('mylove.htm', 'r')
	lrc.lrcCB(file)
#	file = open('search_qinghuaci.htm', 'r')
#	lrcList = lrc.getUrl(file)
#	for item in lrcList:
#		print repr(item)
#		print
