# -*- coding: UTF_8 -*-
#
# Copyright (C) 2009 - Phyware Chen
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2, or (at your option)
# any later version.
#
# The Rhythmbox authors hereby grant permission for non-GPL compatible
# GStreamer plugins to be used and distributed together with GStreamer
# and Rhythmbox. This permission is above and beyond the permissions granted
# by the GPL license by which Rhythmbox is covered. If you modify this code
# you may extend this exception to your version of the code, but you are not
# obligated to do so. If you do not wish to do so, delete this exception
# statement from your version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA.

import re
import codecs

class ParseLrc:
	def __init__(self):
		self.clearList()
		#self.headList = []  #只有保存其他非时间点对应位置
		#self.posList = []   #保存time
		self.patOffset = re.compile(r" *\[offset:([+-]{0,1}[0-9]+)\]")	#pattern of offset
		self.patTime = re.compile(r"^ *\[(?P<min>[0-9]{2}):(?P<sec>[0-9]{2})(\.(?P<milli>[0-9]{2,3})){0,1}\]")
		self.patSub = re.compile(r"\r{0,1}\n{0,1}$")
	
		self.encodings = ['gbk', 'UTF-8', 'gb2312']

	def clearList(self):
		self.fileName = None
		self.lrcList = []  #保存歌词
		self.timeList = []  #保存时间点time及对应位置歌词在lrcTuple中的position
		self.offset = 0

	def parseLrc(self, fileName, fileDuration):
		self.clearList()

		self.fileName = fileName
		self.fileDuration = fileDuration	#in seconds

		file = None
		for item in self.encodings:
			print item
			file = codecs.open(self.fileName, encoding = item, mode = 'r')
			try:
				for line in file:
					pass
			except:
				file = None
				continue
			else:
				break
		if(file == None):
			return False
		file.seek(0)

		isLrcFile = False		#is this file a lrc one

		for line in file:
			isLrc = False # is lrc this line
			lrcListLen = len(self.lrcList)

			while True:
				result = self.patTime.search(line)
				if result == None:
					break;
				isLrc = True	#if not break, this is a lrc line
				isLrcFile = True
				time = self.calcTime(result.group('min'), result.group('sec'), result.group('milli'))
				self.timeList.append([time, lrcListLen])
				line = 	line[result.end():]

			if isLrc == True:
				line = self.patSub.sub('', line)	#remove '\r\n' '\r' '\n'
				self.lrcList.append(line)
			else:
				result = self.patOffset.search(line)
				if result != None:
			 		self.offset = int (result.group(1))
					
		self.timeList.sort()
		for item in self.timeList:
			item[0] = item[0] - self.offset	

		lrcListLen = len(self.lrcList)
		self.lrcList.append('')
		self.timeList.insert(0, [0, lrcListLen])

		self.index = 0
		return isLrcFile
		
	def calcTime(self, min, sec, milli):
		if milli != None:
			milli = int(milli) * (3 - len(milli))
		else:
			milli = 0
		time = (int(min)*60 + int(sec))* 1000 +int(milli)   
		return time

	def getLrc(self):
		return [self.lrcList[item[1]] for item in self.timeList]

	def listLocPos(self, item): #locate position of item,return (index, elapsed time of this line, total time of this line)
		elasped, total = None, None
		timeListLen = len(self.timeList)

		while self.index >= 0 and self.index < timeListLen:	#search between 0 and len(self.timeList) - 1
			if(self.index == timeListLen - 1 and item >= self.timeList[self.index][0]):
				break
			elif(item >= self.timeList[self.index][0] and item < self.timeList[self.index + 1][0]):	#前一句False,则and后一句不判断,故不会出现当self.index == timeListLen - 1时越界情况
				break;
			elif(item > self.timeList[self.index][0]):
				self.index = self.index + 1
			else:
				self.index = self.index - 1

		elapsed = item - self.timeList[self.index][0]
		if self.index < timeListLen - 1:
			total = self.timeList[self.index + 1][0] - self.timeList[self.index][0]
		else:
			total = self.fileDuration * 1000 - self.timeList[self.index][0]
		
		return [self.index, elapsed, total]

	def posToTime(self, index, elapsed, total):
		timeListLen = len(self.timeList)
		tTotal = None	#time total
		if index < timeListLen - 1:
			tTotal = self.timeList[index + 1][0] - self.timeList[index][0]
		else:
			tTotal = self.fileDuration * 1000 - self.timeList[index][0]

		time = self.timeList[index][0] + int( float(elapsed)/float(total) * tTotal)	#in milliseconds
		return int(round(time/1000))	#in seconds

	def printout(self):
		for item in self.timeList:
			print item[0] + self.offset, self.lrcList[item[1]]

if __name__ == '__main__':
	import sys
	lrc = ParseLrc()
	lrc.parseLrc(sys.argv[1], int(sys.argv[2]))
	#lrc.printout()
	res = lrc.listLocPos(int(sys.argv[3]))
	print res[0],res[1]
	#ll = lrc.getLrc()
	#for item in ll:
	#	print item
