# -*- coding: UTF_8 -*- 
#
# Copyright (C) 2009 - Phyware Chen
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2, or (at your option)
# any later version.
#
# The Rhythmbox authors hereby grant permission for non-GPL compatible
# GStreamer plugins to be used and distributed together with GStreamer
# and Rhythmbox. This permission is above and beyond the permissions granted
# by the GPL license by which Rhythmbox is covered. If you modify this code
# you may extend this exception to your version of the code, but you are not
# obligated to do so. If you do not wish to do so, delete this exception
# statement from your version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA.

import gtk
import pango

class UI:
	def __init__(self, plugin):
		self.plugin = plugin	#keep a reference to plugin __init__

		self.spacing = int(self.plugin.getValue('LineSpacing'))
		font = self.plugin.getValue('Font')	#字体,字号
		curColor = self.plugin.getValue('CurColor')	#当前歌词行颜色
		otherColor = self.plugin.getValue('OtherColor')	#其他行歌词颜色

		self.curMarkup = '<span font_desc="' + font + '" foreground="' + curColor + '">placeholderplaceholderplaceholderplaceholderplaceholderplaceholderplaceholderplaceholderplaceholderplaceholderplaceholderplaceholderplaceholderplaceholderplaceholder</span>'
		self.otherMarkup = '<span font_desc="' + font + '" foreground="' + otherColor + '">placeholderplaceholderplaceholderplaceholderplaceholderplaceholderplaceholderplaceholderplaceholderplaceholderplaceholderplaceholderplaceholderplaceholderplaceholder</span>'
		self.curAttr = pango.parse_markup(self.curMarkup)[0]
		self.otherAttr = pango.parse_markup(self.otherMarkup)[0]	#other font attribute

		self.current = None	#current index

		self.validNum = 0	#valid children number,initialized to 0,not 1 to prevent from being dragged
		self.children = None
		self.yScroll = 0
		self.anyLrc = False	#have got any lrc?

		window = gtk.Window()
		self.window = window
		window.connect("delete-event", self.deleteEventCB)
		window.set_default_size(300, 400)
		window.connect("configure-event", self.resize)
		window.set_keep_above(self.plugin.getValue('StayAtTop') == 'True')
		window.set_title('LyricBoxen')
		
		vbox = gtk.VBox()
		self.vbox = vbox
		vbox.set_homogeneous(True)
		vbox.set_spacing(self.spacing)
			
		label = gtk.Label('')	#at least one label to show the current action
		label.set_attributes(self.otherAttr)
		vbox.pack_start(label)	

		layout = gtk.Layout()
		self.layout = layout
		layout.put(vbox, 0, 0)

		layout.add_events(gtk.gdk.BUTTON_RELEASE_MASK | gtk.gdk.BUTTON1_MOTION_MASK | gtk.gdk.POINTER_MOTION_HINT_MASK)
		layout.connect("button-press-event", self.pressCB)
		layout.connect("motion-notify-event", self.moveCB)
		layout.connect("button-release-event", self.releaseCB)

		hbox = gtk.HBox()
		self.label = gtk.Label("<")
		hbox.pack_start(layout)
		hbox.pack_start(self.label, False, False)
		
		self.dragging = False
		self.dragBeginPos = [0, 0]

		window.add(hbox)
		window.show_all()

		self.label.hide()
		layout.window.set_cursor(gtk.gdk.Cursor(gtk.gdk.HAND1))
		colorStr = self.plugin.getValue('BackgroundColor')
		if colorStr != None:
			color = gtk.gdk.color_parse(colorStr)
			self.window.modify_bg(gtk.STATE_NORMAL, color)
			self.layout.modify_bg(gtk.STATE_NORMAL, color)

		self.children = vbox.get_children()
	
		self.showState('LyricBoxen')

	def pressCB(self, widget, event):
		if self.anyLrc == False:	#while there is not any lrc,return.So self.dragging doesn't change
			return False
		if event.button != 1:
			return False 

		self.dragBeginPos = [event.x, event.y]
		self.dragging = True
		self.label.show()
		return True

	def moveCB(self, widget, event):
		if self.dragging == False:	#no any lrc
			return False

		min = 0
		max = (self.children[0].allocation.height + self.spacing) * self.validNum
		if(self.dragging == True):
			dy = event.y - self.dragBeginPos[1]
			toPos = self.yScroll - int(dy)

			if toPos < min or toPos > max :
				return False

			self.layout.window.scroll(0, int(dy))
			self.yScroll = self.yScroll - int(dy)

			height = self.children[0].allocation.height + self.spacing
			previous = self.current
			while(not(self.yScroll >= height * self.current and self.yScroll < height * (self.current + 1))):
				if self.yScroll > height * self.current:
					self.current = self.current + 1
				else:
					self.current = self.current -1
			if previous != self.current:
				self.children[previous].set_attributes(self.otherAttr)
				self.children[self.current].set_attributes(self.curAttr)	
		return True

	def releaseCB(self, widget, event):
		if self.dragging == False:
			return False

		self.label.hide()
		labelHeight = self.children[0].allocation.height
		index = None

		for index in range(self.validNum - 1, -1, -1):
			if self.yScroll >= (labelHeight + self.spacing) * index :
				break
		
		elapsed = self.yScroll - (labelHeight + self.spacing) * index
		self.plugin.setPlayingTime(index, elapsed, labelHeight + self.spacing)
		self.dragging = False	#Whatever:To be put below setPlayingTime maybe better,in order to enable self.update() after that
		return True

	def resize(self, widget, event):
		size = self.window.get_size() 
		sizeWidth = (size[0] - self.vbox.size_request()[0])/2
		sizeHeight = size[1] /2
		self.layout.set_size(self.layout.allocation.width, self.vbox.size_request()[1] + size[1] + self.spacing)
		self.layout.move(self.vbox, sizeWidth, sizeHeight + self.spacing/2) # horizontally at the middle,vertically below the middle
	
	def destroyMyself(self):
		self.window.destroy()

	def resetLrc(self):
		for label in self.children:
			label.set_text('')
		self.validNum = 0
		self.layout.window.scroll(0, self.yScroll)
		self.yScroll = 0
		self.resize(None, None)
		self.anyLrc = False

		if self.current != None :
			self.children[self.current].set_attributes(self.otherAttr)
		self.current = 0

	def setLrcList(self, lrcList):
		index = 0
		lrcListLen = len(lrcList)

		for label in self.children:
			label.set_text(lrcList[index])
			index = index + 1
			if index >= lrcListLen:
				break

		while index < lrcListLen:	#pack remaining lrc
			label = gtk.Label(lrcList[index])
			label.set_attributes(self.otherAttr)
			self.vbox.pack_start(label)
			index = index + 1

		self.validNum = lrcListLen
		self.children = self.vbox.get_children()
		for label in self.children:
			label.show()

		self.resize(None, None)
		self.anyLrc = True
	
	def update(self, index, elapsed, total):
		if self.dragging == True:
			return False

		print index, elapsed, total

		if self.current != index:
			if self.current != None :
				self.children[self.current].set_attributes(self.otherAttr)
			self.children[index].set_attributes(self.curAttr)
			self.current = index

		scrollPos =  int((index + float(elapsed) / float(total)) * (self.children[0].allocation.height + self.spacing))
		scrollNeeded = scrollPos - self.yScroll 
		self.layout.window.scroll(0, -scrollNeeded)
		self.yScroll = scrollPos
	
	def showState(self, text):
		self.children[0].set_text(text)
		self.resize(None, None)

	def showTitle(self, text):
		self.window.set_title(text)
	
	def deleteEventCB(self, widget, event):
		self.plugin.toggleMainUI()
		return True

	def showWindow(self):
		self.window.show()
		
	def hideWindow(self):
		self.window.hide()

if  __name__ == "__main__":
	UI()
	gtk.main()
