# -*- coding: UTF-8 -*- 
#
# Copyright (C) 2009 - Phyware Chen
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2, or (at your option)
# any later version.
#
# The Rhythmbox authors hereby grant permission for non-GPL compatible
# GStreamer plugins to be used and distributed together with GStreamer
# and Rhythmbox. This permission is above and beyond the permissions granted
# by the GPL license by which Rhythmbox is covered. If you modify this code
# you may extend this exception to your version of the code, but you are not
# obligated to do so. If you do not wish to do so, delete this exception
# statement from your version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA.

#TODO:connect to 'elapsed-nano-changed',rather than 'elapsed-changed'.The former signal has been availabel since rhythmbox version 0.12.3,see rhythmbox change-log

import glib
import rb
import rhythmdb
import gtk
import os.path

import GetLrc
import ParseLrc
import UI
import ParseConf

class LyricBoxen(rb.Plugin):
	def __init__(self):
		rb.Plugin.__init__(self)

	def activate(self, shell):
		self.canInit = True	#whether activate is successful

		#get configuration
		self.confParser = ParseConf.ParseConf()
		fileName = self.find_file('lyricboxen.conf')
		self.confParser.parseFile(fileName)
		
		self.dir = self.getValue('LyricDir')
		self.dir = os.path.expanduser(self.dir)
		if os.path.exists(self.dir) == False:
			try:
				os.makedirs(self.dir)
			except:
				self.canInit = False
				return

		self.shell = shell
		self.player = self.shell.props.shell_player
		self.db = self.shell.props.db
		self.uiManager = self.shell.get_ui_manager()

		self.lrcFetcher = GetLrc.GetLrc(self)
		self.lrcParser = ParseLrc.ParseLrc()
		self.ui = UI.UI(self)

		self.idSongChanged = self.player.connect("playing-song-changed", self.songChangedCB)
		self.idPlayingSongProChanged = self.player.connect('playing-song-property-changed', self.playingSongProChangedCB)
		self.idElapsedChanged = None
		entry = self.player.get_playing_entry()
		if entry != None:
			self.songChangedCB(self.player, entry)	#initialized
			del entry
			#self.db.entry_unref(entry)
		
                #add ui to menu view
		uiMainStr = """
		<ui>
		  <menubar name="MenuBar">
		        <menu name="ViewMenu" action="View">
		          <menuitem name="ToggleLyricBoxen" action="ToggleLyricBoxen"/>
		        </menu>
		  </menubar>
		</ui>
		"""

		lyricsBoxenEntrys = (('ToggleLyricBoxen', None, '_LyricBoxen Window', '<Ctrl>l', 'Show LyricBoxen window',self.showUIWindow, True), )
		group = gtk.ActionGroup('LyricBoxen')
		self.group = group
		group.add_toggle_actions(lyricsBoxenEntrys)
		self.uiManager.insert_action_group(group, 0)
		self.idMainUI = self.uiManager.add_ui_from_string(uiMainStr)	#main window ui
		self.uiManager.ensure_update()

		if self.getValue('ShowWindow') == 'False':
			self.toggleMainUI()

	def deactivate(self, shell):
		if self.canInit == False:
			del self.confParser
			del self.dir
			del self.canInit
			return

		del self.canInit
		del self.dir

		if self.idSongChanged != None:
			self.player.disconnect(self.idSongChanged)
		if self.idElapsedChanged != None:
			self.player.disconnect(self.idElapsedChanged)
		self.player.disconnect(self.idPlayingSongProChanged)
		del self.idSongChanged
		del self.idElapsedChanged
		del self.idPlayingSongProChanged

		self.uiManager.remove_ui(self.idMainUI)
		self.uiManager.remove_action_group(self.group)
		del self.group

		del self.shell
		del self.player
		del self.db
		del self.uiManager
		del self.confParser

		del self.lrcFetcher
		del self.lrcParser
		self.ui.destroyMyself()
		del self.ui
		
	def songChangedCB(self, player, entry):
		self.ui.resetLrc()	#UI should reset itself here

		if self.idElapsedChanged != None:
			self.player.disconnect(self.idElapsedChanged)
			self.idElapsedChanged = None

		if entry == None:
			return False

		self.searchLrc()
		return False

	def lrcGot(self, fileName):	#the lrc is got,return isLrcFile
		if fileName == None:
			return False
		isLrcFile =  self.lrcParser.parseLrc(fileName, self.duration)
		if isLrcFile == False:
			return False

		self.ui.setLrcList(self.lrcParser.getLrc())
		self.ui.resize(None, None)
		self.idElapsedChanged = self.player.connect("elapsed-changed", self.elapsedChangedCB)
		return True
	
	def elapsedChangedCB(self, player, elapsed):
		try:				#using this is better,because sometime elapsed = 0 when you drag the lrc
			elapsed = self.player.get_playing_time()
		except glib.GError:
			return False
		(index, elapsed, total) = self.lrcParser.listLocPos(elapsed * 1000)	#seconds -> milliseconds
		self.ui.update(index, elapsed, total)
		return False

	def setPlayingTime(self, index, elapsed, total):
		time = self.lrcParser.posToTime(index, elapsed, total)
		self.player.set_playing_time(time)

	def showState(self, text):
		self.ui.showState(text)

	def toggleMainUI(self):
		action = self.group.get_action('ToggleLyricBoxen')
		action.set_active(not action.get_active())

	def showUIWindow(self, toggleAction):
		if toggleAction.get_active() == True:
			self.ui.showWindow()
		else:
			self.ui.hideWindow()

	def get_url(self, url, callback):
		rb.Loader().get_url(url, callback)

	def getValue(self, key):
		return self.confParser.getValue(key)

	def playingSongProChangedCB(self, player, uri, property, old, newvalue):
		if not(property in ['title', 'artist']):
			return False
		self.searchLrc()
		return False

	def searchLrc(self):
		entry = self.player.get_playing_entry()
		artist = self.db.entry_get(entry, rhythmdb.PROP_ARTIST)
		title = self.db.entry_get(entry, rhythmdb.PROP_TITLE)
		duration = self.db.entry_get(entry, rhythmdb.PROP_DURATION)
		self.duration = duration

		self.ui.showTitle(title + ' - ' + artist)

		self.lrcFetcher.get(title, artist, duration)
